"""Device implementations for the DeviceProtocol interface.

This package provides concrete implementations of DeviceProtocol:
- ADBDevice: Local ADB subprocess calls
- MockDevice: State machine driven mock for testing
- RemoteDevice: HTTP client for remote device agents

Example:
    >>> from AutoGLM_GUI.devices import ADBDevice, RemoteDevice, get_device_manager
    >>>
    >>> # Local ADB device
    >>> device = ADBDevice("emulator-5554")
    >>> device.tap(100, 200)
    >>>
    >>> # Remote device via HTTP
    >>> remote = RemoteDevice("phone_001", "http://device-agent:8001")
    >>> remote.tap(100, 200)
"""

from AutoGLM_GUI.devices.adb_device import ADBDevice, ADBDeviceManager
from AutoGLM_GUI.devices.mock_device import MockDevice
from AutoGLM_GUI.devices.remote_device import RemoteDevice, RemoteDeviceManager

_device_manager: "ADBDeviceManager | None" = None


def get_device_manager() -> ADBDeviceManager:
    """Get the global device manager instance."""
    global _device_manager
    if _device_manager is None:
        _device_manager = ADBDeviceManager()
    return _device_manager


def set_device_manager(manager: "ADBDeviceManager") -> None:
    """Set the global device manager instance (useful for testing)."""
    global _device_manager
    _device_manager = manager


__all__ = [
    "ADBDevice",
    "ADBDeviceManager",
    "MockDevice",
    "RemoteDevice",
    "RemoteDeviceManager",
    "get_device_manager",
    "set_device_manager",
]



"""Scheduled task data models."""

from dataclasses import dataclass, field
from datetime import datetime
from uuid import uuid4


@dataclass
class ScheduledTask:
    """定时任务定义."""

    id: str = field(default_factory=lambda: str(uuid4()))

    # 基础信息
    name: str = ""  # 任务名称
    workflow_uuid: str = ""  # 关联的 Workflow UUID
    device_serialno: str = ""  # 绑定的设备 serialno

    # 调度配置
    cron_expression: str = ""  # Cron 表达式 (如 "0 8 * * *")
    enabled: bool = True  # 是否启用

    # 元数据
    created_at: datetime = field(default_factory=datetime.now)
    updated_at: datetime = field(default_factory=datetime.now)

    # 最近执行信息（只记录最后一次）
    last_run_time: datetime | None = None
    last_run_success: bool | None = None
    last_run_message: str | None = None

    def to_dict(self) -> dict:
        """转换为可序列化的字典."""
        return {
            "id": self.id,
            "name": self.name,
            "workflow_uuid": self.workflow_uuid,
            "device_serialno": self.device_serialno,
            "cron_expression": self.cron_expression,
            "enabled": self.enabled,
            "created_at": self.created_at.isoformat(),
            "updated_at": self.updated_at.isoformat(),
            "last_run_time": self.last_run_time.isoformat()
            if self.last_run_time
            else None,
            "last_run_success": self.last_run_success,
            "last_run_message": self.last_run_message,
        }

    @classmethod
    def from_dict(cls, data: dict) -> "ScheduledTask":
        """从字典创建实例."""
        return cls(
            id=data.get("id", str(uuid4())),
            name=data.get("name", ""),
            workflow_uuid=data.get("workflow_uuid", ""),
            device_serialno=data.get("device_serialno", ""),
            cron_expression=data.get("cron_expression", ""),
            enabled=data.get("enabled", True),
            created_at=datetime.fromisoformat(data["created_at"])
            if data.get("created_at")
            else datetime.now(),
            updated_at=datetime.fromisoformat(data["updated_at"])
            if data.get("updated_at")
            else datetime.now(),
            last_run_time=datetime.fromisoformat(data["last_run_time"])
            if data.get("last_run_time")
            else None,
            last_run_success=data.get("last_run_success"),
            last_run_message=data.get("last_run_message"),
        )



"""Agent factory for creating different agent implementations.

This module provides a factory pattern + registry for creating agents,
making it easy to add new agent types without modifying existing code.
"""

from __future__ import annotations

from typing import Callable, Dict

from AutoGLM_GUI.config import AgentConfig, ModelConfig
from AutoGLM_GUI.logger import logger
from AutoGLM_GUI.types import AgentSpecificConfig

from .protocols import AsyncAgent, BaseAgent


# Agent registry: agent_type -> (creator_function, config_schema)
AGENT_REGISTRY: Dict[str, Callable] = {}


def register_agent(
    agent_type: str,
    creator: Callable,
) -> None:
    """
    Register a new agent type.

    Args:
        agent_type: Unique identifier for the agent type (e.g., "glm", "mai")
        creator: Function that creates the agent instance.
                  Signature: (model_config, agent_config, agent_specific_config, callbacks) -> BaseAgent

    Example:
        >>> def create_mai_agent(model_config, agent_config, mai_config, callbacks):
        >>>     return MAIAgentAdapter(...)
        >>>
        >>> register_agent("mai", create_mai_agent)
    """
    if agent_type in AGENT_REGISTRY:
        logger.warning(f"Agent type '{agent_type}' already registered, overwriting")

    AGENT_REGISTRY[agent_type] = creator
    logger.info(f"Registered agent type: {agent_type}")


def create_agent(
    agent_type: str,
    model_config: ModelConfig,
    agent_config: AgentConfig,
    agent_specific_config: AgentSpecificConfig,
    device,
    takeover_callback: Callable | None = None,
    confirmation_callback: Callable | None = None,
) -> AsyncAgent | BaseAgent:
    """
    Create an agent instance using the factory pattern.

    Args:
        agent_type: Type of agent to create (e.g., "glm", "mai")
        model_config: Model configuration
        agent_config: Agent configuration
        agent_specific_config: Agent-specific configuration (e.g., MAIConfig fields)
        device: DeviceProtocol instance (provided by PhoneAgentManager)
        takeover_callback: Takeover callback
        confirmation_callback: Confirmation callback

    Returns:
        Agent instance implementing AsyncAgent or BaseAgent interface.
        Use runtime type detection (e.g., inspect.iscoroutinefunction) to determine which.

    Raises:
        ValueError: If agent_type is not registered
    """
    if agent_type not in AGENT_REGISTRY:
        available = ", ".join(AGENT_REGISTRY.keys())
        raise ValueError(
            f"Unknown agent type: '{agent_type}'. Available types: {available}"
        )

    creator = AGENT_REGISTRY[agent_type]

    try:
        agent = creator(
            model_config=model_config,
            agent_config=agent_config,
            agent_specific_config=agent_specific_config,
            device=device,
            takeover_callback=takeover_callback,
            confirmation_callback=confirmation_callback,
        )
        logger.debug(f"Created agent of type '{agent_type}'")
        return agent
    except Exception as e:
        logger.error(f"Failed to create agent of type '{agent_type}': {e}")
        raise


def list_agent_types() -> list[str]:
    """Get list of registered agent types."""
    return list(AGENT_REGISTRY.keys())


def is_agent_type_registered(agent_type: str) -> bool:
    """Check if an agent type is registered."""
    return agent_type in AGENT_REGISTRY


# ==================== Built-in Agent Creators ====================


def _create_async_glm_agent(
    model_config: ModelConfig,
    agent_config: AgentConfig,
    agent_specific_config: AgentSpecificConfig,  # noqa: ARG001
    device,
    takeover_callback: Callable | None = None,
    confirmation_callback: Callable | None = None,
) -> AsyncAgent:
    """Create AsyncGLMAgent instance.

    This is the default async implementation that supports:
    - Native streaming with AsyncIterator
    - Immediate cancellation with asyncio.CancelledError
    - No worker threads or queues needed

    Note: 'glm' now uses AsyncGLMAgent by default for better performance.
    Use 'glm-sync' if you need the old synchronous implementation.
    """
    from .glm.async_agent import AsyncGLMAgent

    # Note: AsyncGLMAgent implements AsyncAgent Protocol, but pyright cannot verify
    # async generator function compatibility with Protocol. This is a known limitation
    # of Python's type system. The implementation is correct at runtime.
    return AsyncGLMAgent(  # type: ignore[return-value]
        model_config=model_config,
        agent_config=agent_config,
        device=device,
        confirmation_callback=confirmation_callback,
        takeover_callback=takeover_callback,
    )


def _create_glm_agent_sync(
    model_config: ModelConfig,
    agent_config: AgentConfig,
    agent_specific_config: AgentSpecificConfig,  # noqa: ARG001
    device,
    takeover_callback: Callable | None = None,
    confirmation_callback: Callable | None = None,
) -> BaseAgent:
    """Create synchronous GLMAgent (legacy, for backward compatibility).

    This is the old synchronous implementation using AgentStepStreamer.
    Only use this if you have compatibility issues with AsyncGLMAgent.
    """
    from .glm.agent import GLMAgent

    return GLMAgent(
        model_config=model_config,
        agent_config=agent_config,
        device=device,
        confirmation_callback=confirmation_callback,
        takeover_callback=takeover_callback,
    )


def _create_internal_mai_agent(
    model_config: ModelConfig,
    agent_config: AgentConfig,
    agent_specific_config: AgentSpecificConfig,
    device,
    takeover_callback: Callable | None = None,
    confirmation_callback: Callable | None = None,
) -> BaseAgent:
    from .mai.agent import InternalMAIAgent

    history_n = agent_specific_config.get("history_n", 3)

    return InternalMAIAgent(
        model_config=model_config,
        agent_config=agent_config,
        device=device,
        history_n=history_n,
        confirmation_callback=confirmation_callback,
        takeover_callback=takeover_callback,
    )


register_agent("glm", _create_glm_agent_sync)  # 默认使用同步实现 (向后兼容)
register_agent("glm-async", _create_async_glm_agent)  # 异步实现 (显式选择)
register_agent("async-glm", _create_async_glm_agent)  # 别名
register_agent("mai", _create_internal_mai_agent)



"""Data models for AutoGLM-GUI."""

from AutoGLM_GUI.models.history import ConversationRecord, DeviceHistory
from AutoGLM_GUI.models.scheduled_task import ScheduledTask

__all__ = [
    "ConversationRecord",
    "DeviceHistory",
    "ScheduledTask",
]



"""Scheduled task manager with APScheduler."""

import json
from datetime import datetime
from pathlib import Path
from typing import Optional

from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.triggers.cron import CronTrigger

from AutoGLM_GUI.logger import logger
from AutoGLM_GUI.models.scheduled_task import ScheduledTask


class SchedulerManager:
    _instance: Optional["SchedulerManager"] = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self):
        if hasattr(self, "_initialized"):
            return
        self._initialized = True
        self._tasks_path = Path.home() / ".config" / "autoglm" / "scheduled_tasks.json"
        self._scheduler = BackgroundScheduler()
        self._tasks: dict[str, ScheduledTask] = {}
        self._file_mtime: Optional[float] = None

    def start(self) -> None:
        self._load_tasks()
        for task in self._tasks.values():
            if task.enabled:
                self._add_job(task)
        self._scheduler.start()
        logger.info(f"SchedulerManager started with {len(self._tasks)} task(s)")

    def shutdown(self) -> None:
        self._scheduler.shutdown(wait=False)
        logger.info("SchedulerManager shutdown")

    def create_task(
        self,
        name: str,
        workflow_uuid: str,
        device_serialno: str,
        cron_expression: str,
        enabled: bool = True,
    ) -> ScheduledTask:
        task = ScheduledTask(
            name=name,
            workflow_uuid=workflow_uuid,
            device_serialno=device_serialno,
            cron_expression=cron_expression,
            enabled=enabled,
        )
        self._tasks[task.id] = task
        self._save_tasks()

        if enabled:
            self._add_job(task)

        logger.info(f"Created scheduled task: {name} (id={task.id})")
        return task

    def update_task(self, task_id: str, **kwargs) -> Optional[ScheduledTask]:
        task = self._tasks.get(task_id)
        if not task:
            return None

        old_enabled = task.enabled
        old_cron = task.cron_expression

        for key, value in kwargs.items():
            if value is not None and hasattr(task, key):
                setattr(task, key, value)

        task.updated_at = datetime.now()
        self._save_tasks()

        if old_enabled and not task.enabled:
            self._remove_job(task_id)
        elif not old_enabled and task.enabled:
            self._add_job(task)
        elif task.enabled and old_cron != task.cron_expression:
            self._remove_job(task_id)
            self._add_job(task)

        logger.info(f"Updated scheduled task: {task.name} (id={task_id})")
        return task

    def delete_task(self, task_id: str) -> bool:
        task = self._tasks.pop(task_id, None)
        if not task:
            return False

        self._remove_job(task_id)
        self._save_tasks()
        logger.info(f"Deleted scheduled task: {task.name} (id={task_id})")
        return True

    def list_tasks(self) -> list[ScheduledTask]:
        return list(self._tasks.values())

    def get_task(self, task_id: str) -> Optional[ScheduledTask]:
        return self._tasks.get(task_id)

    def set_enabled(self, task_id: str, enabled: bool) -> bool:
        task = self._tasks.get(task_id)
        if not task:
            return False

        if task.enabled == enabled:
            return True

        task.enabled = enabled
        task.updated_at = datetime.now()
        self._save_tasks()

        if enabled:
            self._add_job(task)
        else:
            self._remove_job(task_id)

        logger.info(f"{'Enabled' if enabled else 'Disabled'} task: {task.name}")
        return True

    def get_next_run_time(self, task_id: str) -> Optional[datetime]:
        job = self._scheduler.get_job(task_id)
        if job and job.next_run_time:
            return job.next_run_time.replace(tzinfo=None)
        return None

    def _add_job(self, task: ScheduledTask) -> None:
        try:
            parts = task.cron_expression.split()
            if len(parts) != 5:
                logger.error(f"Invalid cron expression: {task.cron_expression}")
                return

            trigger = CronTrigger(
                minute=parts[0],
                hour=parts[1],
                day=parts[2],
                month=parts[3],
                day_of_week=parts[4],
            )

            self._scheduler.add_job(
                self._execute_task,
                trigger=trigger,
                id=task.id,
                args=[task.id],
                replace_existing=True,
            )
            logger.debug(f"Added job for task: {task.name}")
        except Exception as e:
            logger.error(f"Failed to add job for task {task.name}: {e}")

    def _remove_job(self, task_id: str) -> None:
        try:
            if self._scheduler.get_job(task_id):
                self._scheduler.remove_job(task_id)
                logger.debug(f"Removed job: {task_id}")
        except Exception as e:
            logger.warning(f"Failed to remove job {task_id}: {e}")

    def _execute_task(self, task_id: str) -> None:
        task = self._tasks.get(task_id)
        if not task:
            logger.warning(f"Task {task_id} not found for execution")
            return

        logger.info(f"Executing scheduled task: {task.name}")

        from AutoGLM_GUI.device_manager import DeviceManager
        from AutoGLM_GUI.history_manager import history_manager
        from AutoGLM_GUI.models.history import ConversationRecord, MessageRecord
        from AutoGLM_GUI.phone_agent_manager import PhoneAgentManager
        from AutoGLM_GUI.workflow_manager import workflow_manager

        workflow = workflow_manager.get_workflow(task.workflow_uuid)
        if not workflow:
            self._record_failure(task, "Workflow not found")
            return

        device_manager = DeviceManager.get_instance()
        device = None
        for d in device_manager.get_devices():
            if d.serial == task.device_serialno and d.state.value == "online":
                device = d
                break

        if not device:
            self._record_failure(task, "Device offline")
            return

        manager = PhoneAgentManager.get_instance()
        acquired = manager.acquire_device(
            device.primary_device_id,
            timeout=0,
            raise_on_timeout=False,
            auto_initialize=True,
        )

        if not acquired:
            self._record_failure(task, "Device busy")
            return

        start_time = datetime.now()

        # 收集完整对话消息
        messages: list[MessageRecord] = []
        messages.append(
            MessageRecord(
                role="user",
                content=workflow["text"],
                timestamp=start_time,
            )
        )

        try:
            agent = manager.get_agent(device.primary_device_id)
            agent.reset()

            # 使用 step 循环执行，收集每步信息
            is_first = True
            result_message = ""
            task_success = False

            while agent.step_count < agent.agent_config.max_steps:
                step_result = agent.step(workflow["text"] if is_first else None)  # type: ignore[misc]
                is_first = False

                # 收集每个 step 的消息
                messages.append(
                    MessageRecord(
                        role="assistant",
                        content="",
                        timestamp=datetime.now(),
                        thinking=step_result.thinking,  # type: ignore[union-attr]
                        action=step_result.action,  # type: ignore[union-attr]
                        step=agent.step_count,
                    )
                )

                if step_result.finished:  # type: ignore[union-attr]
                    result_message = step_result.message or "Task completed"  # type: ignore[union-attr]
                    task_success = step_result.success  # type: ignore[union-attr]
                    break
            else:
                result_message = "Max steps reached"
                task_success = False

            steps = agent.step_count
            end_time = datetime.now()

            record = ConversationRecord(
                task_text=workflow["text"],
                final_message=result_message,
                success=task_success,
                steps=steps,
                start_time=start_time,
                end_time=end_time,
                duration_ms=int((end_time - start_time).total_seconds() * 1000),
                source="scheduled",
                source_detail=task.name,
                error_message=None if task_success else result_message,
                messages=messages,
            )
            history_manager.add_record(task.device_serialno, record)

            if task_success:
                self._record_success(task, result_message)
            else:
                self._record_failure(task, result_message)

        except Exception as e:
            end_time = datetime.now()
            error_msg = str(e)
            logger.error(f"Scheduled task failed: {task.name} - {error_msg}")

            record = ConversationRecord(
                task_text=workflow["text"],
                final_message=error_msg,
                success=False,
                steps=0,
                start_time=start_time,
                end_time=end_time,
                duration_ms=int((end_time - start_time).total_seconds() * 1000),
                source="scheduled",
                source_detail=task.name,
                error_message=error_msg,
                messages=messages,
            )
            history_manager.add_record(task.device_serialno, record)

            self._record_failure(task, error_msg)

        finally:
            manager.release_device(device.primary_device_id)

    def _record_success(self, task: ScheduledTask, message: str) -> None:
        task.last_run_time = datetime.now()
        task.last_run_success = True
        task.last_run_message = message[:500] if message else ""
        self._save_tasks()
        logger.info(f"Scheduled task completed: {task.name}")

    def _record_failure(self, task: ScheduledTask, error: str) -> None:
        task.last_run_time = datetime.now()
        task.last_run_success = False
        task.last_run_message = error[:500] if error else ""
        self._save_tasks()
        logger.warning(f"Scheduled task failed: {task.name} - {error}")

    def _load_tasks(self) -> None:
        if not self._tasks_path.exists():
            return

        try:
            with open(self._tasks_path, encoding="utf-8") as f:
                data = json.load(f)
            tasks_data = data.get("tasks", [])
            self._tasks = {t["id"]: ScheduledTask.from_dict(t) for t in tasks_data}
            self._file_mtime = self._tasks_path.stat().st_mtime
            logger.debug(f"Loaded {len(self._tasks)} scheduled tasks")
        except Exception as e:
            logger.warning(f"Failed to load scheduled tasks: {e}")

    def _save_tasks(self) -> None:
        self._tasks_path.parent.mkdir(parents=True, exist_ok=True)
        temp_path = self._tasks_path.with_suffix(".tmp")

        try:
            data = {"tasks": [t.to_dict() for t in self._tasks.values()]}
            with open(temp_path, "w", encoding="utf-8") as f:
                json.dump(data, f, indent=2, ensure_ascii=False)
            temp_path.replace(self._tasks_path)
            self._file_mtime = self._tasks_path.stat().st_mtime
            logger.debug(f"Saved {len(self._tasks)} scheduled tasks")
        except Exception as e:
            logger.error(f"Failed to save scheduled tasks: {e}")
            if temp_path.exists():
                temp_path.unlink()


scheduler_manager = SchedulerManager()



import json
import traceback
from typing import Any, Callable

from openai import OpenAI

from AutoGLM_GUI.actions import ActionHandler, ActionResult
from AutoGLM_GUI.config import AgentConfig, ModelConfig, StepResult
from AutoGLM_GUI.device_protocol import DeviceProtocol
from AutoGLM_GUI.logger import logger
from AutoGLM_GUI.prompt_config import get_messages, get_system_prompt

from .message_builder import MessageBuilder
from .parser import GLMParser


class GLMAgent:
    def __init__(
        self,
        model_config: ModelConfig,
        agent_config: AgentConfig,
        device: DeviceProtocol,
        confirmation_callback: Callable[[str], bool] | None = None,
        takeover_callback: Callable[[str], None] | None = None,
        thinking_callback: Callable[[str], None] | None = None,
    ):
        self.model_config = model_config
        self.agent_config = agent_config

        self.openai_client = OpenAI(
            base_url=model_config.base_url,
            api_key=model_config.api_key,
            timeout=120,
        )
        self.parser = GLMParser()

        self.device = device
        self.action_handler = ActionHandler(
            device=self.device,
            confirmation_callback=confirmation_callback,
            takeover_callback=takeover_callback,
        )

        self._context: list[dict[str, Any]] = []
        self._step_count = 0
        self._is_running = False
        self._thinking_callback = thinking_callback

    def run(self, task: str) -> str:
        self._context = []
        self._step_count = 0
        self._is_running = True

        try:
            result = self._execute_step(task, is_first=True)

            if result.finished:
                return result.message or "Task completed"

            while self._step_count < self.agent_config.max_steps and self._is_running:
                result = self._execute_step(is_first=False)

                if result.finished:
                    return result.message or "Task completed"

            return "Max steps reached"
        finally:
            self._is_running = False

    def step(self, task: str | None = None) -> StepResult:
        is_first = len(self._context) == 0

        if is_first and not task:
            raise ValueError("Task is required for the first step")

        return self._execute_step(task, is_first)

    def reset(self) -> None:
        self._context = []
        self._step_count = 0
        self._is_running = False

    def abort(self) -> None:
        self._is_running = False
        logger.info("Agent aborted by user")

    def _stream_request(
        self,
        messages: list[dict[str, Any]],
        on_thinking_chunk: Callable[[str], None] | None = None,
    ) -> tuple[str, str, str]:
        stream = self.openai_client.chat.completions.create(
            messages=messages,  # type: ignore[arg-type]
            model=self.model_config.model_name,
            max_tokens=self.model_config.max_tokens,
            temperature=self.model_config.temperature,
            top_p=self.model_config.top_p,
            frequency_penalty=self.model_config.frequency_penalty,
            extra_body=self.model_config.extra_body,
            stream=True,
        )

        raw_content = ""
        buffer = ""
        action_markers = ["finish(message=", "do(action="]
        in_action_phase = False

        for chunk in stream:
            if len(chunk.choices) == 0:
                continue
            if chunk.choices[0].delta.content is not None:
                content = chunk.choices[0].delta.content
                raw_content += content

                if in_action_phase:
                    continue

                buffer += content

                marker_found = False
                for marker in action_markers:
                    if marker in buffer:
                        thinking_part = buffer.split(marker, 1)[0]
                        if on_thinking_chunk:
                            on_thinking_chunk(thinking_part)
                        in_action_phase = True
                        marker_found = True
                        break

                if marker_found:
                    continue

                is_potential_marker = False
                for marker in action_markers:
                    for i in range(1, len(marker)):
                        if buffer.endswith(marker[:i]):
                            is_potential_marker = True
                            break
                    if is_potential_marker:
                        break

                if not is_potential_marker:
                    if on_thinking_chunk:
                        on_thinking_chunk(buffer)
                    buffer = ""

        thinking, action = self._parse_raw_response(raw_content)
        return thinking, action, raw_content

    def _parse_raw_response(self, content: str) -> tuple[str, str]:
        if "finish(message=" in content:
            parts = content.split("finish(message=", 1)
            thinking = parts[0].strip()
            action = "finish(message=" + parts[1]
            return thinking, action

        if "do(action=" in content:
            parts = content.split("do(action=", 1)
            thinking = parts[0].strip()
            action = "do(action=" + parts[1]
            return thinking, action

        if "<answer>" in content:
            parts = content.split("<answer>", 1)
            thinking = parts[0].replace("<think>", "").replace("</think>", "").strip()
            action = parts[1].replace("</answer>", "").strip()
            return thinking, action

        return "", content

    def _execute_step(
        self, user_prompt: str | None = None, is_first: bool = False
    ) -> StepResult:
        self._step_count += 1

        screenshot = self.device.get_screenshot()
        current_app = self.device.get_current_app()

        if is_first:
            system_prompt = self.agent_config.system_prompt
            if system_prompt is None:
                system_prompt = get_system_prompt(self.agent_config.lang)

            self._context.append(MessageBuilder.create_system_message(system_prompt))

            screen_info = MessageBuilder.build_screen_info(current_app)
            text_content = f"{user_prompt}\n\n{screen_info}"

            self._context.append(
                MessageBuilder.create_user_message(
                    text=text_content, image_base64=screenshot.base64_data
                )
            )
        else:
            screen_info = MessageBuilder.build_screen_info(current_app)
            # 如果有新的用户消息（多轮对话场景），把它加入消息中
            if user_prompt:
                text_content = f"{user_prompt}\n\n** Screen Info **\n\n{screen_info}"
            else:
                # 继续执行当前任务，只需要屏幕信息
                text_content = f"** Screen Info **\n\n{screen_info}"

            self._context.append(
                MessageBuilder.create_user_message(
                    text=text_content, image_base64=screenshot.base64_data
                )
            )

        try:
            msgs = get_messages(self.agent_config.lang)
            if self.agent_config.verbose:
                print("\n" + "=" * 50)
                print(f"💭 {msgs['thinking']}:")
                print("-" * 50)

            callback = self._thinking_callback
            if callback is None and self.agent_config.verbose:

                def print_chunk(chunk: str) -> None:
                    print(chunk, end="", flush=True)

                callback = print_chunk

            thinking, action_str, raw_content = self._stream_request(
                self._context, on_thinking_chunk=callback
            )
        except Exception as e:
            if self.agent_config.verbose:
                traceback.print_exc()
            return StepResult(
                success=False,
                finished=True,
                action=None,
                thinking="",
                message=f"Model error: {e}",
            )

        try:
            action = self.parser.parse(action_str)
        except ValueError as e:
            if self.agent_config.verbose:
                logger.warning(f"Failed to parse action: {e}, treating as finish")
            action = {"_metadata": "finish", "message": action_str}

        if self.agent_config.verbose:
            print()
            print("-" * 50)
            print(f"🎯 {msgs['action']}:")
            print(json.dumps(action, ensure_ascii=False, indent=2))
            print("=" * 50 + "\n")

        self._context[-1] = MessageBuilder.remove_images_from_message(self._context[-1])

        try:
            result = self.action_handler.execute(
                action, screenshot.width, screenshot.height
            )
        except Exception as e:
            if self.agent_config.verbose:
                traceback.print_exc()
            result = ActionResult(success=False, should_finish=True, message=str(e))

        self._context.append(
            MessageBuilder.create_assistant_message(
                f"<think>{thinking}</think><answer>{action_str}</answer>"
            )
        )

        finished = action.get("_metadata") == "finish" or result.should_finish

        if finished and self.agent_config.verbose:
            msgs = get_messages(self.agent_config.lang)
            print("\n" + "🎉 " + "=" * 48)
            print(
                f"✅ {msgs['task_completed']}: {result.message or action.get('message', msgs['done'])}"
            )
            print("=" * 50 + "\n")

        return StepResult(
            success=result.success,
            finished=finished,
            action=action,
            thinking=thinking,
            message=result.message or action.get("message"),
        )

    @property
    def context(self) -> list[dict[str, Any]]:
        return self._context.copy()

    @property
    def step_count(self) -> int:
        return self._step_count

    @property
    def is_running(self) -> bool:
        return self._is_running



"""Device control utilities for Android automation."""

import subprocess
import time

from AutoGLM_GUI.adb.apps import APP_PACKAGES
from AutoGLM_GUI.adb.timing import TIMING_CONFIG
from AutoGLM_GUI.platform_utils import build_adb_command


def get_current_app(device_id: str | None = None) -> str:
    adb_prefix = build_adb_command(device_id)

    result = subprocess.run(
        adb_prefix + ["shell", "dumpsys", "window"],
        capture_output=True,
        text=True,
        encoding="utf-8",
    )
    output = result.stdout
    if not output:
        raise ValueError("No output from dumpsys window")

    for line in output.split("\n"):
        if "mCurrentFocus" in line or "mFocusedApp" in line:
            for app_name, package in APP_PACKAGES.items():
                if package in line:
                    return app_name

    return "System Home"


def tap(
    x: int, y: int, device_id: str | None = None, delay: float | None = None
) -> None:
    if delay is None:
        delay = TIMING_CONFIG.device.default_tap_delay

    adb_prefix = build_adb_command(device_id)

    subprocess.run(
        adb_prefix + ["shell", "input", "tap", str(x), str(y)], capture_output=True
    )
    time.sleep(delay)


def double_tap(
    x: int, y: int, device_id: str | None = None, delay: float | None = None
) -> None:
    if delay is None:
        delay = TIMING_CONFIG.device.default_double_tap_delay

    adb_prefix = build_adb_command(device_id)

    subprocess.run(
        adb_prefix + ["shell", "input", "tap", str(x), str(y)], capture_output=True
    )
    time.sleep(TIMING_CONFIG.device.double_tap_interval)
    subprocess.run(
        adb_prefix + ["shell", "input", "tap", str(x), str(y)], capture_output=True
    )
    time.sleep(delay)


def long_press(
    x: int,
    y: int,
    duration_ms: int = 3000,
    device_id: str | None = None,
    delay: float | None = None,
) -> None:
    if delay is None:
        delay = TIMING_CONFIG.device.default_long_press_delay

    adb_prefix = build_adb_command(device_id)

    subprocess.run(
        adb_prefix
        + ["shell", "input", "swipe", str(x), str(y), str(x), str(y), str(duration_ms)],
        capture_output=True,
    )
    time.sleep(delay)


def swipe(
    start_x: int,
    start_y: int,
    end_x: int,
    end_y: int,
    duration_ms: int | None = None,
    device_id: str | None = None,
    delay: float | None = None,
) -> None:
    if delay is None:
        delay = TIMING_CONFIG.device.default_swipe_delay

    adb_prefix = build_adb_command(device_id)

    if duration_ms is None:
        dist_sq = (start_x - end_x) ** 2 + (start_y - end_y) ** 2
        duration_ms = int(dist_sq / 1000)
        duration_ms = max(1000, min(duration_ms, 2000))

    subprocess.run(
        adb_prefix
        + [
            "shell",
            "input",
            "swipe",
            str(start_x),
            str(start_y),
            str(end_x),
            str(end_y),
            str(duration_ms),
        ],
        capture_output=True,
    )
    time.sleep(delay)


def back(device_id: str | None = None, delay: float | None = None) -> None:
    if delay is None:
        delay = TIMING_CONFIG.device.default_back_delay

    adb_prefix = build_adb_command(device_id)

    subprocess.run(
        adb_prefix + ["shell", "input", "keyevent", "4"], capture_output=True
    )
    time.sleep(delay)


def home(device_id: str | None = None, delay: float | None = None) -> None:
    if delay is None:
        delay = TIMING_CONFIG.device.default_home_delay

    adb_prefix = build_adb_command(device_id)

    subprocess.run(
        adb_prefix + ["shell", "input", "keyevent", "KEYCODE_HOME"], capture_output=True
    )
    time.sleep(delay)


def launch_app(
    app_name: str, device_id: str | None = None, delay: float | None = None
) -> bool:
    if delay is None:
        delay = TIMING_CONFIG.device.default_launch_delay

    if app_name not in APP_PACKAGES:
        return False

    adb_prefix = build_adb_command(device_id)
    package = APP_PACKAGES[app_name]

    subprocess.run(
        adb_prefix
        + [
            "shell",
            "monkey",
            "-p",
            package,
            "-c",
            "android.intent.category.LAUNCHER",
            "1",
        ],
        capture_output=True,
    )
    time.sleep(delay)
    return True
